//---------------------------------------------------------------------------
// GPGee - GNU Privacy Guard Explorer Extension
// GPGeeConfigure.cpp - Configuration dialog
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of GPGee.
//
// GPGee is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GPGee is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// File Notes:
// The code for GPGee's configuration form.  This is pretty much just a UI
// wrapper around the TConfiguration class.  Yes, I know what I said about
// seperating the UI from functional code in notes for GPGeeSignEncrypt.cpp.
// In this case I wanted to create a "drag-n-drop" configuration class that
// could be reused anywhere.  The functionality in GPGeeSignEncrypt isn't
// likely to be reused, so it'd done quick and dirty like.
//---------------------------------------------------------------------------
#include <vcl.h>
#include <FileCtrl.hpp>
#pragma hdrstop

#include "TConfiguration.h"
#include "GPGeeConfigure.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TformGPGeeConfigure *formGPGeeConfigure;
extern TConfiguration *GPGeeConfig;             // All our configuration data
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Constructor for the form
//
__fastcall TformGPGeeConfigure::TformGPGeeConfigure(TComponent* Owner): TForm(Owner)
{
  char *buffer;

  bFirstActivation = true;
  sGPGProgram = sOptfile = sKeyring = sSecretKeyring = "";

  // Find where the module is sitting and use that as the directory for where the help file is too.
  buffer = new char[256];
  if (!GetModuleFileName(HInstance, buffer, 256))
    sBinDir = "";
  else
    sBinDir = ExtractFileDir(buffer);
  delete buffer;
  Application->HelpFile = sBinDir + "\\GPGee.hlp";
  HelpFile = sBinDir + "\\GPGee.hlp";
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Form OnActivate event handler
//
void __fastcall TformGPGeeConfigure::FormActivate(TObject *Sender)
{
  if (bFirstActivation) {
    FormFirstActivation(Sender);
    bFirstActivation = false;
  }  // if (bFirstActivation)
}  // void __fastcall TformGPGeeConfigure::FormActivate(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Form OnFirstActivation pseudo event handler
//
void __fastcall TformGPGeeConfigure::FormFirstActivation(TObject *Sender)
{
  sGPGProgram = GPGeeConfig->Values["GPG Program"];
  if (!sGPGProgram.IsEmpty())
    MinimizeName(sGPGProgram, edtGPGProgram);
  chbGPGProgram->Checked = !sGPGProgram.IsEmpty() && FileExists(sGPGProgram);

  sOptfile = GPGeeConfig->Values["Options File"];
  if (!sOptfile.IsEmpty())
    MinimizeName(sOptfile, edtOptfile);
  chbOptfile->Checked = !sOptfile.IsEmpty() && FileExists(sOptfile);

  sKeyring = GPGeeConfig->Values["Keyring"];
  if (!sKeyring.IsEmpty())
    MinimizeName(sKeyring, edtKeyring);
  chbKeyring->Checked = !sKeyring.IsEmpty() && FileExists(sKeyring);

  sSecretKeyring = GPGeeConfig->Values["Secret Keyring"];
  if (!sSecretKeyring.IsEmpty())
    MinimizeName(sSecretKeyring, edtSecretKeyring);
  chbSecretKeyring->Checked = !sSecretKeyring.IsEmpty() && FileExists(sSecretKeyring);

  chbForceTrust->Checked = GPGeeConfig->Values["Force Trust"];

  SanityCheck();
}  // void __fastcall TformGPGeeConfigure::FormFirstActivation(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// btnGPGProgram OnClick handler - get the location of the GPG binary
//
void __fastcall TformGPGeeConfigure::btnGPGProgramClick(TObject *Sender)
{
  OpenDialog->Title = "Select location of gpg executable...";
  OpenDialog->DefaultExt = "exe";
  OpenDialog->Filter = "GnuPG executable (gpg.exe)|gpg.exe|All files (*.*)|*.*";
  OpenDialog->InitialDir = "";

  if (!sGPGProgram.IsEmpty() && FileExists(sGPGProgram))
    OpenDialog->FileName = sGPGProgram;
  else
    OpenDialog->FileName = GPGeeConfig->Values["GPGProgram"];

  if (OpenDialog->FileName.IsEmpty())
    OpenDialog->InitialDir = GPGeeConfig->Values["ProgramFilesDir"];
  if (OpenDialog->Execute()) {
    sGPGProgram = OpenDialog->FileName;
    MinimizeName(sGPGProgram, edtGPGProgram);
  }  // if (OpenDialog->Execute())
  SanityCheck();
}  // void __fastcall TformGPGeeConfigure::btnGPGProgramClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// chbGPGProgram OnClick event handler.  If the user clicks to enable setting
// the program path then do the right thing
//
void __fastcall TformGPGeeConfigure::chbGPGProgramClick(TObject *Sender)
{
  if (chbGPGProgram->Checked && (sGPGProgram.IsEmpty() || !FileExists(sGPGProgram)))
    btnGPGProgramClick(Sender);
  else
    SanityCheck();
}  // void __fastcall TformGPGeeConfigure::chbGPGProgramClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// btnOptfile OnClick handler - get the location of the GPG options file
//
void __fastcall TformGPGeeConfigure::btnOptfileClick(TObject *Sender)
{
  OpenDialog->Title = "Select location of gpg options file...";
  OpenDialog->DefaultExt = "conf";
  OpenDialog->Filter = "GnuPG configuration file (gpg.conf)|gpg.conf|Conf files (*.conf)|*.conf|All files (*.*)|*.*";
  OpenDialog->InitialDir = "";
  OpenDialog->FileName = "";

  if (!sOptfile.IsEmpty() && FileExists(sOptfile))
    OpenDialog->FileName = sOptfile;
  else if (chbGPGProgram->Checked)
    OpenDialog->InitialDir = ExtractFileDir(sGPGProgram);
  else
    OpenDialog->FileName = GPGeeConfig->Values["OptFile"];
  if (OpenDialog->FileName.IsEmpty() && OpenDialog->InitialDir.IsEmpty())
    OpenDialog->InitialDir = GPGeeConfig->Values["HomeDir"];
  if (OpenDialog->InitialDir.IsEmpty())
    OpenDialog->InitialDir = ExtractFileDir(GPGeeConfig->Values["gpgProgram"]);

  if (OpenDialog->Execute()) {
    sOptfile = OpenDialog->FileName;
    MinimizeName(sOptfile, edtOptfile);
  }  // if (OpenDialog->Execute())
  SanityCheck();
}  // void __fastcall TformGPGeeConfigure::btnOptfileClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// chbOptfile OnClick event handler
//
void __fastcall TformGPGeeConfigure::chbOptfileClick(TObject *Sender)
{
  if (chbOptfile->Checked && (sOptfile.IsEmpty() || !FileExists(sOptfile)))
    btnOptfileClick(Sender);
  else
    SanityCheck();
}  // void __fastcall TformGPGeeConfigure::chbOptfileClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// btnKeyring OnClick handler - get the location of the public key keyring
//
void __fastcall TformGPGeeConfigure::btnKeyringClick(TObject *Sender)
{
  OpenDialog->Title = "Select location of gpg public-key keyring...";
  OpenDialog->DefaultExt = "gpg";
  OpenDialog->Filter = "GnuPG public keyring (pubring.gpg)|pubring.gpg|GnuPG files (*.gpg)|*.gpg|All files (*.*)|*.*";
  OpenDialog->FileName = "";
  OpenDialog->InitialDir = "";

  if (!sKeyring.IsEmpty() && FileExists(sKeyring))
    OpenDialog->FileName = sKeyring;
  else
    OpenDialog->InitialDir = GPGeeConfig->Values["HomeDir"];

  if (OpenDialog->Execute()) {
    sKeyring = OpenDialog->FileName;
    MinimizeName(sKeyring, edtKeyring);
  }  // if (OpenDialog->Execute())
  SanityCheck();
}  // void __fastcall TformGPGeeConfigure::btnKeyringClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// chbKeyring OnClick event handler
//
void __fastcall TformGPGeeConfigure::chbKeyringClick(TObject *Sender)
{
  if (chbKeyring->Checked && (sKeyring.IsEmpty() || !FileExists(sKeyring)))
    btnKeyringClick(Sender);
  else
    SanityCheck();
}  // void __fastcall TformGPGeeConfigure::chbKeyringClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// btnSecretKeyring OnClick handler - get the location of the secret key
// keyring
//
void __fastcall TformGPGeeConfigure::btnSecretKeyringClick(TObject *Sender)
{
  OpenDialog->Title = "Select location of gpg secret-key keyring...";
  OpenDialog->DefaultExt = "gpg";
  OpenDialog->Filter = "GnuPG secret keyring (secring.gpg)|secring.gpg|GnuPG files (*.gpg)|*.gpg|All files (*.*)|*.*";
  OpenDialog->FileName = "";
  OpenDialog->InitialDir = "";

  if (!sSecretKeyring.IsEmpty() && FileExists(sSecretKeyring))
    OpenDialog->FileName = sSecretKeyring;
  else
    OpenDialog->InitialDir = GPGeeConfig->Values["HomeDir"];

  if (OpenDialog->Execute()) {
    sSecretKeyring = OpenDialog->FileName;
    MinimizeName(sSecretKeyring, edtSecretKeyring);
  }  // if (OpenDialog->Execute())
  SanityCheck();
}  // void __fastcall TformGPGeeConfigure::btnSecretKeyringClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// chbSecretKeyring OnClick event handler
//
void __fastcall TformGPGeeConfigure::chbSecretKeyringClick(TObject *Sender)
{
  if (chbSecretKeyring->Checked && (sSecretKeyring.IsEmpty() || !FileExists(sSecretKeyring)))
    btnSecretKeyringClick(Sender);
  else
    SanityCheck();
}  // void __fastcall TformGPGeeConfigure::btnSecretKeyringClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Take a filename and reduce it so that it fits inside the space given on
// an edit box.
AnsiString __fastcall TformGPGeeConfigure::MinimizeName(AnsiString FileName, TEdit *Edit)
{
  TCanvas *FileNameCanvas = new TCanvas;
  HWND Dummy;

  FileNameCanvas->Handle = GetDeviceContext(Dummy);
  FileNameCanvas->Font->Assign(Edit->Font);
  Edit->Text = Filectrl::MinimizeName(FileName, FileNameCanvas, Edit->Width);
  return Edit->Text;
}  // AnsiString __fastcall TformGPGeeConfigure::MinimizeName(AnsiString FileName, TControl *Control)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Check to see if the settings on the form make sense and attempt to
// intelligently adjust any that don't
//
void __fastcall TformGPGeeConfigure::SanityCheck(void)
{
  chbGPGProgram->Checked = chbGPGProgram->Checked && !sGPGProgram.IsEmpty() && FileExists(sGPGProgram);
  chbOptfile->Checked = chbOptfile->Checked && !sOptfile.IsEmpty() && FileExists(sOptfile);
  chbKeyring->Checked = chbKeyring->Checked && !sKeyring.IsEmpty() && FileExists(sKeyring);
  chbSecretKeyring->Checked = chbSecretKeyring->Checked && !sSecretKeyring.IsEmpty() && FileExists(sSecretKeyring);
  edtGPGProgram->Enabled = chbGPGProgram->Checked;
  btnGPGProgram->Enabled = chbGPGProgram->Checked;
  edtOptfile->Enabled = chbOptfile->Checked;
  btnOptfile->Enabled = chbOptfile->Checked;
  edtKeyring->Enabled = chbKeyring->Checked;
  btnKeyring->Enabled = chbKeyring->Checked;
  edtSecretKeyring->Enabled = chbSecretKeyring->Checked;
  btnSecretKeyring->Enabled = chbSecretKeyring->Checked;
}  // void __fastcall TformGPGeeConfigure::SanityCheck(void)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// The user has done the deed and clicked ok...
//
void __fastcall TformGPGeeConfigure::btnOkClick(TObject *Sender)
{
  if (chbGPGProgram->Checked)
    GPGeeConfig->Values["GPG Program"] = sGPGProgram;
  if (chbOptfile->Checked)
    GPGeeConfig->Values["Options File"] = sOptfile;
  if (chbKeyring->Checked)
    GPGeeConfig->Values["Keyring"] = sKeyring;
  if (chbSecretKeyring->Checked)
    GPGeeConfig->Values["Secret Keyring"] = sSecretKeyring;
  GPGeeConfig->Values["Force Trust"] = chbForceTrust->Checked;
  ModalResult = mrOk;
}  // void __fastcall TformGPGeeConfigure::btnOkClick(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// User clicked on the "Help" button
//
void __fastcall TformGPGeeConfigure::btnHelpClick(TObject *Sender)
{
  Application->HelpContext(3);
}
//---------------------------------------------------------------------------

