//---------------------------------------------------------------------------
// GPGee - GNU Privacy Guard Explorer Extension
// GPGeePassPhrase.cpp - Passphrase dialog
// Copyright 2005, Kurt Fitzner <kfitzner@excelcia.org>
//---------------------------------------------------------------------------
// This file is part of GPGee.
//
// GPGee is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// GPGee is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "GPGeePassPhrase.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "ThemeMgr"
#pragma resource "*.dfm"
TformGPGeePassPhrase *formGPGeePassPhrase;

//---------------------------------------------------------------------------
// Form constructor
//
__fastcall TformGPGeePassPhrase::TformGPGeePassPhrase(TComponent* Owner): TForm(Owner)
{
  bFirstActivation = true;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// The form's OnActivate event handler
//
void __fastcall TformGPGeePassPhrase::FormActivate(TObject *Sender)
{
  if (bFirstActivation) {
    bFirstActivation = false;
    FormActivateFirst(Sender);
  }  // if (bFirstActivation)
}  // void __fastcall TformGPGeePassPhrase::FormActivate(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//  There is no OnFirstActivation event, so we fake it
//
void __fastcall TformGPGeePassPhrase::FormActivateFirst(TObject *Sender)
{
  if (!sKeyUser.IsEmpty()) {
    pnlSymmetric->Width = 0;
    pnlSymmetric->Height = 0;
    pnlSymmetric->Visible = false;
    edtUser->Text = sKeyUser;
    edtId->Text   = sKeyId;
    edtType->Text = sKeyType;
    edtSize->Text = sKeySize;
    edtDate->Text = sKeyCreationDate;
  }  // if (!sKeyUser.IsEmpty())
}  // void __fastcall TformGPGeePassPhrase::FormActivateFirst(TObject *Sender)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// chbHideTyping OnClick event handler
// Hide or unhide the passphrase
void __fastcall TformGPGeePassPhrase::chbHideTypingClick(TObject *Sender)
{
  edtPassphrase->PasswordChar = chbHideTyping->Checked?'*':'\0';
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// edtPassphrase OnChange event handler
// Turn on or off the Ok button depending on whether there is text in the
// passphrase control.
void __fastcall TformGPGeePassPhrase::edtPassphraseChange(TObject *Sender)
{
  btnPPOk->Enabled = !edtPassphrase->Text.IsEmpty();
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Interface function to pull up the form preloaded with the info about the
// key that the system is asking for the passphrase to unlock
//
int __fastcall TformGPGeePassPhrase::GetPassphrase(AnsiString User, AnsiString Id, AnsiString Type, AnsiString Size, AnsiString Date)
{
  sKeyUser = User;
  sKeyId = Id;
  sKeyType = Type;
  sKeySize = Size;
  sKeyCreationDate = Date;
  return ShowModal();
}  // int __fastcall TformGPGeePassPhrase::GetPassphrase(AnsiString User, AnsiString Id, AnsiString Type, AnsiString Size, AnsiString Date)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Interface function for when we want to get a passphrase for symmetric
// encryption.
//
int __fastcall TformGPGeePassPhrase::GetSymmetricPassphrase(void)
{
  sKeyUser = "";
  return ShowModal();
}  // int __fastcall TformGPGeePassPhrase::GetSymmetricPassphrase(void)



